import { useState, useRef, useCallback, useEffect, } from "react"

/*
  - uri: string
  - mode:
    - null  - default, fetch data ONCE
    - interval_sec
    - interval_stop
*/

export default function usePolling(uri, onResponce, mode = null) {
    const [isPolling, setPolling] = useState(false);

    const initialPollRef = useRef(true);
    const initialPoll = initialPollRef.current;
    
    const intervalTimerIdRef = useRef(null);
    const intervalTimerId = intervalTimerIdRef.current;


    const pollData = useCallback(() => {
        setPolling(true);
        initialPollRef.current = false;

        fetch(uri)
            .then((responce) => {
                setPolling(false);

                if (typeof mode?.interval_sec === 'number') {
                    console.log("Schedule", uri, "fetch in", mode.interval_sec, "sec");
                    intervalTimerIdRef.current = setTimeout(pollData, mode.interval_sec * 1000);
                }

                return responce.json();
            })
            .then((json) => {
                onResponce(json);
            })
            .catch((err) => {
                console.warn(err.message);
            })
    }, [uri, mode, onResponce, initialPollRef, intervalTimerIdRef]);

    const stopPollInterval = useCallback(() => {
        console.log("Cancel scheduled fetch for", uri);
        clearTimeout(intervalTimerId);
        intervalTimerIdRef.current = null;
        initialPollRef.current = true;
    }, [uri, intervalTimerId, intervalTimerIdRef, initialPollRef]);

    useEffect(() => {
        if ((initialPoll || (typeof mode?.interval_sec === 'number' && intervalTimerId === null)) && !isPolling) {
            pollData();
        } else 
        if (mode?.interval_stop && intervalTimerId) {
            stopPollInterval();
        }
    }, [initialPoll, mode, intervalTimerId, isPolling, pollData, stopPollInterval]);

    return isPolling;
}